#pragma once
#include <vector>
#include <string>

UENUM(meta = (Bitflags, UseEnumValuesAsMaskValuesInEditor = "true"))
enum HandSide
{
	//HSnone = 0			UMETA(Hidden),
	Left	= 1 << 0    UMETA(DisplayName = "Left")	,
	Right	= 1 << 1    UMETA(DisplayName = "Right"),
};
ENUM_RANGE_BY_VALUES(HandSide, HandSide::Left, HandSide::Right);


UENUM(meta = (Bitflags, UseEnumValuesAsMaskValuesInEditor = "true"))
enum ActuationPoint
{
	//APnone	= 0			UMETA(Hidden),
	Thumb	= 1 << 0	UMETA(DisplayName = "Thumb"),
	Index	= 1 << 1	UMETA(DisplayName = "Index"),
	Middle	= 1 << 2	UMETA(DisplayName = "Middle"),
	Palm	= 1 << 3	UMETA(DisplayName = "Palm"),
};
ENUM_RANGE_BY_VALUES(ActuationPoint, ActuationPoint::Thumb, ActuationPoint::Index, ActuationPoint::Middle, ActuationPoint::Palm);

enum class TrackingType{
	DEFAULT,	//!< Deprecated, contains only closure values 
	WEART_HAND	//!< Tracking with closures, and abduction value for thumb
};

enum HandClosingState
{
	Open = 0,
	Closing = 1,
	Closed = 2
};


enum GraspingState
{
	Grabbed = 0,
	Released = 1
};


UENUM()
enum class TextureType : uint8
{
	ClickNormal = 0, ClickSoft = 1, DoubleClick = 2,
	AluminiumFineMeshSlow = 3, AluminiumFineMeshFast = 4,
	PlasticMeshSlow = 5, ProfiledAluminiumMeshMedium = 6, ProfiledAluminiumMeshFast = 7,
	RhombAluminiumMeshMedium = 8,
	TextileMeshMedium = 9,
	CrushedRock = 10,
	VenetianGranite = 11,
	SilverOak = 12,
	LaminatedWood = 13,
	ProfiledRubberSlow = 14,
	VelcroHooks = 15,
	VelcroLoops = 16,
	PlasticFoil2 = 17,
	Leather = 18,
	Cotton1 = 19,
	Aluminium = 20,
	DoubleSidedTape = 21
};


// Constants shared by the WeArt components
namespace WeArtConstants
{
	const std::string WEART_SDK_TYPE = "SdkUnreal";
	const std::string WEART_SDK_VERSION = "1.1.0";
	const std::string WEART_SDK_TRACK_TYPE = "TrackType1";

	const float defaultTemperature = 0.5f;
	const float minTemperature = 0.0f;
	const float maxTemperature = 1.0f;

	const float defaultForce = 0.0f;
	const float minForce = 0.0f;
	const float maxForce = 1.0f;

	const float defaultClosure = 0.0f;
	const float minClosure = 0.0f;
	const float maxClosure = 1.0f;

	const float staticTextureVelocity = 0.5f;

	const int defaultTextureIndex = 0;
	const int minTextureIndex = 0;
	const int maxTextureIndex = 254;
	const int nullTextureIndex = 255;

	const float defaultTextureVelocity[3] = { 0.5f, 0.0f, 0.0f };
	const float minTextureVelocity[3] = { 0.0f, 0.0f, 0.0f };
	const float maxTextureVelocity[3] = { 1.0f, 1.0f, 1.0f };

	const float defaultCollisionMultiplier = 20.0f;
	const float minCollisionMultiplier = 0.0f;
	const float maxCollisionMultiplier = 100.0f;

	const float defaultVolumeTexture = 100.0f;
	const float minVolumeTexture = 0.0f;
	const float maxVolumeTexture = 100.0f;

	const float thresholdThumbClosure = 0.5f;
	const float thresholdIndexClosure = 0.5f;
	const float thresholdMiddleClosure = 0.5f;

	const float defaultGraspForce = 0.3f;
	const float dynamicForceSensibility = 10.0f;
};
