#pragma once

#include "CoreMinimal.h"
#include "Sockets.h"
#include "WeArtMessageSerializer.h"
#include "WeArtThimbleTrackingObject.h"
#include "WeArtTrackingCalibration.h"
#include "GameFramework/Actor.h"
#include "Kismet/GameplayStatics.h"
#include <string>

#include "WeArtController.generated.h"

enum WeArtMessageType
{
	MessageSent, MessageReceived
};

enum WeArtError
{
	ConnectionError, SendMessageError, ReceiveMessageError
};

static char messagesSeparator = '~';

/// <summary>
/// Weart controller, used to connect to the Weart middleware, perform operations and receive messages
/// </summary>
UCLASS()
class WEARTPLUGIN_API UWeArtController : public UGameInstanceSubsystem
{
	GENERATED_BODY()

public:

	UWeArtController();
	~UWeArtController();

	AWeArtTrackingCalibration* TrackingCalibration;

	virtual void Initialize(FSubsystemCollectionBase& Collection) override;
	virtual void Deinitialize() override;

	UFUNCTION(BlueprintCallable)
		void PauseController();
	UFUNCTION(BlueprintCallable)
		void UnpauseController();
	UFUNCTION(BlueprintCallable)
		void StartCalibration();
	UFUNCTION(BlueprintCallable)
		void StopCalibration();

	void ScheduleController();

	void SendMessage(WeArtMessage* message);
	bool ReceiveMessages(std::vector<WeArtMessage*>& messages);

	void OnMessageReceived(WeArtMessage* msg);
	void StopConnection();

	void LogMessage(WeArtMessageType type, FString& message);
	void LogError(WeArtError error, FString& message);

	void HandleReceivedMessage(FString& message);
	// We are keeping track of attached thimble tracking objects,
	// so we can forward received messages to them. Usually
	// that could be solved via delegates, but native C++
	// has no support for those, and UE4 delegates would require
	// the WeArtMessage object to be an UCLASS for some reason.
	TArray<UWeArtThimbleTrackingObject*> thimbleTrackingObjects;

protected:

	UPROPERTY()
		int ClientPort;

	UPROPERTY()
		bool DebugMessages;

	UPROPERTY()
		bool ControllerAutoStart;

	UPROPERTY()
		bool CalibrationAutoStart;

	bool foundTrackingCalibration = false;
	bool isCancellationRequested = false;
	FSocket* socket;
	WeArtMessageSerializer messageSerializer;
	uint8* messageReceivedBuffer[1024];
	std::string trailingText;
	bool IsConnected = false;
	bool IsPaused = false;
};
