#include "WeArtDeviceTrackingObject.h"
#include <map>
#include <utility>

std::map<OffsetPreset, std::pair<FVector, FVector>> offsetPresetsMap = {
	{ HtcViveLeft,     std::make_pair(FVector(-0.01f, 0.04f, -0.085f),   FVector(0.0f, -90.0f, -90.0f)) },
	{ HtcViveRight,    std::make_pair(FVector(0.01f, 0.04f, -0.085f),   FVector(0.0f, -90.0f, -90.0f)) },
	{ OculusLeftWrist,  std::make_pair(FVector(-8.0f, -3.0f, 0.0f), FVector(-280.0f, 180.0f, 0.0f)) },
	{ OculusRightWrist, std::make_pair(FVector(-3.0, 0.0f,0.0f), FVector(-100.0f, 180.0f, 0.0f)) },
	{ OculusLeftPalm,  std::make_pair(FVector(1.0f, 0.0f, 0.0f), FVector(-280.0f, 180.0f, 0.0f)) },
	{ OculusRightPalm, std::make_pair(FVector(1.0f, 0.0f,0.0f), FVector(-100.0f, 180.0f, 0.0f)) },
	{ OculusLeftPalmTwisted,  std::make_pair(FVector(0.0f, 0.0f, 0.0f), FVector(-280.0f,-280.0f, 0.0f)) },
	{ OculusRightPalmTwisted, std::make_pair(FVector(3.4f,5.0f,-5.0f), FVector(280.0f, 90.0f, 0.0f)) }
};

UWeArtDeviceTrackingObject::UWeArtDeviceTrackingObject()
{
	// Set this component to be initialized when the game starts, and to be ticked every frame.  You can turn these features
	// off to improve performance if you don't need them.
	PrimaryComponentTick.bCanEverTick = true;
	PrimaryComponentTick.TickGroup = (updateMethod == TrackingUpdateMethod::PrePhysicsUpdate ? TG_PrePhysics : TG_PostPhysics);
}

UWeArtDeviceTrackingObject::~UWeArtDeviceTrackingObject()
{
}

void UWeArtDeviceTrackingObject::TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction)
{
	Super::TickComponent(DeltaTime, TickType, ThisTickFunction);

	if (disableTracking) {
		return;
	}

	if (!trackingSource.IsValid()) {
		return;
	}

	if (!GetOwner()) {
		return;
	}

	auto posOffset = useOffsetPreset ? offsetPresetsMap[offsetPreset].first : positionOffset;
	auto rotOffset = useOffsetPreset ? offsetPresetsMap[offsetPreset].second : rotationOffset;

	FVector newLoc = trackingSource.TransformPosition(posOffset);
	FQuat newRot = trackingSource.TransformRotation(FQuat::MakeFromEuler(rotOffset));
	FTransform newTrans(newRot, newLoc);
	GetOwner()->SetActorTransform(newTrans);
}
