#pragma once
#include "CoreMinimal.h"
#include "Components/ActorComponent.h"
#include "WeArtCommon.h"

#include "WeArtDeviceTrackingObject.generated.h"

UENUM()
enum TrackingUpdateMethod
{
	PrePhysicsUpdate,
	PostPhysicsUpdate
};

UENUM()
enum OffsetPreset
{
	HtcViveLeft,
	HtcViveRight,
	OculusLeftWrist,
	OculusRightWrist,
	OculusLeftPalm,
	OculusRightPalm,
	OculusLeftPalmTwisted,
	OculusRightPalmTwisted
};

UCLASS(ClassGroup = (Custom), Blueprintable, meta = (BlueprintSpawnableComponent))
class WEARTPLUGIN_API UWeArtDeviceTrackingObject : public USceneComponent
{
	GENERATED_BODY()

public:
	// Sets default values for this component's properties
	UWeArtDeviceTrackingObject();
	~UWeArtDeviceTrackingObject();

	virtual void TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction) override;

protected:

	// State variables
	UPROPERTY(EditAnywhere, Category = WeArtDeviceTrackingObject, Meta = (DisplayName = "Update method"))
		TEnumAsByte<TrackingUpdateMethod> updateMethod;
	UPROPERTY(BlueprintReadWrite, Category = WeArtDeviceTrackingObject, Meta = (DisplayName = "Tracking source"))
		FTransform trackingSource;

	// Enable external tracker 
	UPROPERTY(EditAnywhere, Category = WeArtDeviceTrackingObject, Meta = (DisplayName = "Disable Tracking"))
		bool disableTracking;

	// Offset
	UPROPERTY(EditAnywhere)
		bool useOffsetPreset;
	UPROPERTY(EditAnywhere, Category = WeArtDeviceTrackingObject, Meta = (DisplayName = "Offset preset", EditCondition="useOffsetPreset"))
		TEnumAsByte<OffsetPreset> offsetPreset;
	UPROPERTY(EditAnywhere, Category = WeArtDeviceTrackingObject, Meta = (DisplayName = "Position offset", EditCondition = "!useOffsetPreset"))
		FVector positionOffset;
	UPROPERTY(EditAnywhere, Category = WeArtDeviceTrackingObject, Meta = (DisplayName = "Rotation offset", EditCondition = "!useOffsetPreset"))
		FVector rotationOffset;
};
