// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "Components/ActorComponent.h"
#include "WeArtThimbleTrackingObject.h"
#include "WeArtHapticObject.h"
#include "WeArtTouchableObject.h"
#include "WeArtGraspEvents.h"
#include "WeArtHandController.generated.h"

UCLASS( ClassGroup=(Custom), meta=(BlueprintSpawnableComponent) )
class WEARTPLUGIN_API UWeArtHandController : public UActorComponent
{
	GENERATED_BODY()

public:
	// Sets default values for this component's properties
	UWeArtHandController();

	UPROPERTY(EditAnywhere)
		bool IsHandGrabInvisible = true;

	UPROPERTY(BlueprintReadWrite)
		TEnumAsByte<HandSide> handSide;

	UFUNCTION(BlueprintCallable, Category = "WEART HandController")
		void SetIsHandGrabInvisible(bool value);

	UFUNCTION(BlueprintCallable, Category = "WEART HandController")
		bool GetIsHandGrabInvisible();

protected:
	// Called when the game starts
	virtual void BeginPlay() override;

	UFUNCTION(BlueprintCallable)
	void Init(UWeArtThimbleTrackingObject* thumbTracking, UWeArtThimbleTrackingObject* indexTracking, UWeArtThimbleTrackingObject* middleTracking,
		UWeArtHapticObject* thumbHaptic, UWeArtHapticObject* indexHaptic, UWeArtHapticObject* middleHaptic);

	// Overlap handlers. Unity's equivalent of "StayOverlapped" is handled by the tick event.
	// Hit events are not currently handled at all, being more specific than their Unity counterparts
	// and not really suitable for our use.
	UFUNCTION()
		void BeginOverlap(AActor* MyOverlappedActor, AActor* OtherActor);
	UFUNCTION()
		void EndOverlap(AActor* MyOverlappedActor, AActor* OtherActor);

	void CheckHandClosingState(void);
	void HandleGraspingState(void);

	UWeArtThimbleTrackingObject* thumbTrackingObject;
	UWeArtThimbleTrackingObject* indexTrackingObject;
	UWeArtThimbleTrackingObject* middleTrackingObject;
	UWeArtHapticObject* thumbHapticObject;
	UWeArtHapticObject* indexHapticObject;
	UWeArtHapticObject* middleHapticObject;

	UWeArtTouchableObject* touchedObjectByThumb;
	UWeArtTouchableObject* touchedObjectByIndex;
	UWeArtTouchableObject* touchedObjectByMiddle;
	UWeArtTouchableObject* touchedObject;

	HandClosingState closingState = HandClosingState::Open;
	GraspingState graspingState = GraspingState::Released;
	bool initialized = false;

public:
	// Called every frame
	virtual void TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction) override;
	AWeArtGraspEvents* weArtGraspEvents;
	bool foundGraspEvents;
	

};
