#include "WeArtHapticObject.h"
#include "WeArtController.h"
#include "Kismet/GameplayStatics.h"
#include <algorithm>
#include <mutex>


UWeArtHapticObject::UWeArtHapticObject() : activeEffects()
{
	PrimaryComponentTick.bCanEverTick = true;
}


void UWeArtHapticObject::AddEffect(WeArtEffect* effect)
{
	if (!ContainsEffect(effect))
	{
		activeEffects.push_back(effect);
		UpdateEffects();
	}
}


void UWeArtHapticObject::RemoveEffect(WeArtEffect* effect)
{
	if (ContainsEffect(effect))
	{
		activeEffects.erase(std::remove(activeEffects.begin(), activeEffects.end(), effect), activeEffects.end());
		UpdateEffects();
	}
}


bool UWeArtHapticObject::ContainsEffect(WeArtEffect* effect)
{
	return std::find(activeEffects.begin(), activeEffects.end(), effect) != activeEffects.end();
}

void UWeArtHapticObject::RemoveLastEffect() 
{
	activeEffects.pop_back();
}

void UWeArtHapticObject::ClearEffects() 
{
	activeEffects.clear();
}


void UWeArtHapticObject::UpdateEffects(void)
{
	if (activeEffects.empty()) {
		weArtForce.value = WeArtConstants::defaultForce;
		weArtForce.active = false;
		weArtTemperature.value = WeArtConstants::defaultTemperature;
		weArtTemperature.active = false;
		weArtTexture.textureType = (TextureType)WeArtConstants::defaultTextureIndex;
		weArtTexture.textureVelocity = WeArtConstants::defaultTextureVelocity[2];
		weArtTexture.active = false;

		StopTemperatureMessage msg1;
		SendMessage(&msg1);
		StopForceMessage msg2;
		SendMessage(&msg2);
		StopTextureMessage msg3;
		SendMessage(&msg3);
	}
	else {
		// Update temperature and send relevant messages
		WeArtEffect* tempEffect = nullptr;

		for (std::vector<WeArtEffect*>::reverse_iterator i = activeEffects.rbegin(); i != activeEffects.rend(); ++i) {
			if ((*i)->getTemperature().active)
			{
				tempEffect = *i;
				break;
			}
		}
		FWeArtTemperature newTemp = tempEffect != nullptr ? tempEffect->getTemperature() : FWeArtTemperature();
		if (!(newTemp == weArtTemperature)) {
			if (!newTemp.active) {
				StopTemperatureMessage msg;
				SendMessage(&msg);
			}
			else {
				SetTemperatureMessage msg(newTemp.value);
				SendMessage(&msg);
			}
			weArtTemperature = newTemp;
		}

		// Update force and set relevant messages
		WeArtEffect* forceEffect = nullptr;
		for (std::vector<WeArtEffect*>::reverse_iterator i = activeEffects.rbegin(); i != activeEffects.rend(); ++i) {
			if ((*i)->getForce().active)
			{
				forceEffect = *i;
				break;
			}
		}
		FWeArtForce newForce = forceEffect != nullptr ? forceEffect->getForce() : FWeArtForce();
		if (!(newForce == weArtForce)) {
			if (!newForce.active) {
				StopForceMessage msg;
				SendMessage(&msg);
			}
			else {
				const float fValue[3] = { newForce.value, 0.0, 0.0 };
				SetForceMessage msg(fValue);
				SendMessage(&msg);
			}
			weArtForce = newForce;
		}

		// Update texture and set relevant messages
		WeArtEffect* texEffect = nullptr;
		for (std::vector<WeArtEffect*>::reverse_iterator i = activeEffects.rbegin(); i != activeEffects.rend(); ++i) {
			if ((*i)->getTexture().active)
			{
				texEffect = *i;
				break;
			}
		}

		FWeArtTexture newTex = texEffect != nullptr ? texEffect->getTexture() : FWeArtTexture();
		if (!(newTex == weArtTexture)) {
			if (!newTex.active) {
				StopTextureMessage msg;
				SendMessage(&msg);
			}
			else {
				const float texValue[3] = { WeArtConstants::defaultTextureVelocity[0], WeArtConstants::defaultTextureVelocity[1], newTex.textureVelocity };
				SetTextureMessage msg((int)newTex.textureType, texValue, newTex.volume);
				SendMessage(&msg);
			}
			weArtTexture = newTex;
		}
	}
}

void UWeArtHapticObject::SendMessage(WeArtMessage* msg)
{
	UGameInstance* GameInstance = GetOuter()->GetWorld()->GetGameInstance();
	UWeArtController* weArtController = GameInstance->GetSubsystem<UWeArtController>();
	if (!weArtController) {
		return;
	}

	for (HandSide hs : TEnumRange<HandSide>()) {
		if (hs & handSideFlag) {
			for (ActuationPoint ap : TEnumRange<ActuationPoint>()) {
				if (ap & actuationPointFlag) {
					msg->setHandSide(hs);
					msg->setActuationPoint(ap);
					weArtController->SendMessage(msg);
				}
			};
		}
	};
}
