#pragma once

#include "CoreMinimal.h"
#include "Components/ActorComponent.h"
#include "WeArtCommon.h"
#include "WeArtEffect.h"
#include <vector>

// Included cause of a bug in UnrealHeaderTool.
// Those are already included in WeArtEffect.h,
// but need to be included there as well.
#include "WeArtTemperature.h"
#include "WeArtForce.h"
#include "WeArtTexture.h"

#include "WeArtHapticObject.generated.h"

// A haptic object class, representing player-controller objects able to interact
// with objects of the @TouchableObject class.
//
// The communication with the lower layers is triggered by adding and removing effects
// (see @AddEffect() and @RemoveEffect()), which subsequently triggers an @UpdateEffects() event.
//
// Such event then sends foward (to, presumably, the middleware)
// messages representing the processed events
UCLASS(ClassGroup = (Custom), Blueprintable, meta = (BlueprintSpawnableComponent))
class WEARTPLUGIN_API UWeArtHapticObject : public UActorComponent
{
	GENERATED_BODY()

public:
	// Sets default values for this component's properties
	UWeArtHapticObject();

protected:

	// Hand/finger state variables
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = WeArtHapticObject, meta = (Bitmask, BitmaskEnum = HandSide))
		int32 handSideFlag;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = WeArtHapticObject, meta = (Bitmask, BitmaskEnum = ActuationPoint))
		int32 actuationPointFlag;

	// State variables. Serialized.
	UPROPERTY(EditAnywhere, Meta = (DisplayName = "Temperature"))
		FWeArtTemperature weArtTemperature;
	UPROPERTY(EditAnywhere, Meta = (DisplayName = "Force"))
		FWeArtForce weArtForce;
	UPROPERTY(EditAnywhere, Meta = (DisplayName = "Texture"))
		FWeArtTexture weArtTexture;

	// List of effect active on this object
	// Not serialized.
	std::vector<WeArtEffect*> activeEffects;

public:

	void AddEffect(WeArtEffect* effect);
	void RemoveEffect(WeArtEffect* effect);
	void RemoveLastEffect();
	void ClearEffects();
	bool ContainsEffect(WeArtEffect* effect);
	void UpdateEffects(void);

	void SendMessage(WeArtMessage* msg);
};
