#pragma once

#include "CoreMinimal.h"
#include "Components/ActorComponent.h"
#include "WeArtCommon.h"
#include "WeArtMessages.h"

#include "WeArtThimbleTrackingObject.generated.h"

/// <summary>
/// Object used to track a single thimble object and its closure value
/// </summary>
UCLASS(ClassGroup = (Custom), Blueprintable, meta = (BlueprintSpawnableComponent))
class WEARTPLUGIN_API UWeArtThimbleTrackingObject : public UActorComponent
{
	GENERATED_BODY()

public:
	// Sets default values for this component's properties
	UWeArtThimbleTrackingObject();
	~UWeArtThimbleTrackingObject();

	virtual void BeginPlay() override;
	virtual void EndPlay(const EEndPlayReason::Type EndPlayReason) override;

	void OnMessageReceived(WeArtMessage *msg);

	inline float GetClosure() const
	{
		return Closure;
	};

	inline float GetAbduction() const
	{
		return Abduction;
	};


protected:

	// Hand/finger state variables
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = WeArtThimbleTrackingObject)
		TEnumAsByte<HandSide> handSide;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = WeArtThimbleTrackingObject)
		TEnumAsByte<ActuationPoint> actuationPoint;

	UPROPERTY(VisibleAnywhere, BlueprintReadWrite, Category = WeArtThimbleTrackingObject)
		float Closure;

	UPROPERTY(VisibleAnywhere, BlueprintReadWrite, Category = WeArtThimbleTrackingObject)
		float Abduction;
};

