#pragma once

#include "CoreMinimal.h"
#include "WeArtCommon.h"
#include "WeArtTexture.generated.h"

/// <summary>
/// Texture information to be applied to an effect
/// </summary>
USTRUCT()
struct FWeArtTexture {
	GENERATED_BODY()

	UPROPERTY(EditAnywhere)
		bool active;
	UPROPERTY(EditAnywhere, Meta = (DisplayName = "Index", EditCondition = "active", ClampMin = "0", ClampMax = "21"))
		TextureType textureType;
	UPROPERTY(EditAnywhere, Meta = (DisplayName = "ForcedVelocity", EditCondition = "active"))
		bool isForcedTextureVelocity = false;

	// The value 0 represents no texture feedback and value 1 the maximum intensity.
		float textureVelocity = 0.0f;

	UPROPERTY(NoClear, Meta = (DisplayName = "Volume", EditCondition = "active", ClampMin = "0.0", ClampMax = "1.0"))
		float volume;

	FWeArtTexture() : active(false),
		textureType((TextureType)WeArtConstants::defaultTextureIndex),
		isForcedTextureVelocity(false),
		textureVelocity(WeArtConstants::defaultTextureVelocity[2]),
		volume(WeArtConstants::defaultVolumeTexture)
	{};

	bool operator==(const FWeArtTexture& other) const {
		return (active == other.active && textureType == other.textureType && textureVelocity == other.textureVelocity && volume == other.volume && isForcedTextureVelocity == other.isForcedTextureVelocity);
	};
};

